#include <iostream>
#include <cstring>
#include <string>
#include <fstream>
#include <iomanip>
#include <sstream>
#include <sys/time.h>
#include <sys/stat.h>
#include <getopt.h> 
#include <massiveDIS.hh>
#include <cstdlib>

using namespace std;



bool dynscale = false;

void print_usage() {
  cout << endl
       << "Usage:" << endl
       << " -x  --x <x>             set x Bjorken" << endl
       << " -Q  --Q <Q>             set Q" << endl
       << " -d  --dynamic-scale     use scale mu=Q+2mc (default mu=Q)" << endl
       << endl;
  exit(0);
  return;
}

void read_arguments(int argc, char* argv[], double &x, double &Q) {
  const char* const short_options = "hx:Q:d";
  const struct option long_options[] = { { "help", 0, NULL, 'h' },
					 { "x", 1, NULL, 'x' },
           { "Q", 1, NULL, 'Q' },
					 { "dynamic-scale", 0, NULL, 'd' },
					 { NULL, 0, NULL, 0 } };
  int next_option;
  ostringstream sset;
  do {
    next_option = getopt_long (argc, argv, short_options, long_options, NULL);
    switch (next_option) {
    case 'h':
      print_usage();
    case 'x':
      x = strtod(optarg, NULL);
      break;
    case 'Q':
      Q = strtod(optarg, NULL);
      break;
    case 'd':
      dynscale = true;
      break;
    case '?':
      print_usage();
    case -1: break;
    default: abort();
    }
  }
  while (next_option != -1);
  return;
}


/**
 * Main program
 */
int main(int argc, char* argv[])
{

  if(argc==1) cout << endl << "\033[0;31m Hint: Run with -h to access options\033[0m" << endl << endl;

  
  struct timeval t0, t1;
  gettimeofday(&t0,NULL);


  // default values
  double x = 0.01;
  double Q = 10;

  // read runtime arguments
  read_arguments(argc, argv, x, Q);

  // print parameters
  cout << endl;
  cout << "Bjorken x: " << x << endl;
  cout << "Q: "         << Q << " GeV" << endl << endl;

  // set the charm as the massive parton
  parton part = c;

  // PDF sets for Fixed Order and Resummed results
  string PDFsetFO = "NNPDF30_nlo_as_0118_nf_3";
  string PDFset   = "NNPDF30_nlo_as_0118";
  int member = 0;

  // Initialize massiveDIS for Fixed Order and Resummed (use different PDFs)
  // available processes:  DISNC, DISNCGAMMA, DISNCZ, DISNCZGAMMA, DISCC
  MassiveDIS disFO(DISCC, part, PDFsetFO, member);
  MassiveDIS dis  (DISCC, part, PDFset, member);

  // get charm mass
  double mc = dis.GetPDF()->getQuarkMass(4);

  // set factorization scale
  double muFQ = 1.;
  if(dynscale) muFQ = sqrt(1 + mc*mc/Q/Q);

  // print results
  // available structure functions: F_1, F_2, F_3, F_L
  sfunction fun = F_2;  
  string sfun[] = {"FLc", "F1c", "F2c", "F3c"}; 
  cout << sfun[(int) fun] + " Fixed Order (3FS)     = " <<  disFO.F(x, Q, NLO, FO,    fun, muFQ)  << endl;
  cout << sfun[(int) fun] + " F2c FONLL-A (FLNR)    = " <<  dis  .F(x, Q, NLO, FONLL, fun, muFQ)  << endl;
  cout << sfun[(int) fun] + " F2c ACOT = newFONLL-A = " <<  dis  .F(x, Q, NLO, ACOT,  fun, muFQ)  << endl;
  cout << sfun[(int) fun] + " F2c ZeroMass          = " <<  dis  .F(x, Q, NLO, ZM,    fun, muFQ)  << endl << endl;

  cout << sfun[(int) fun] + " DeltaIC               = " <<  dis  .DeltaIC(x, Q, NLO,     fun, muFQ)  << endl << endl;

  gettimeofday(&t1,NULL);
  double t=t1.tv_sec-t0.tv_sec+(t1.tv_usec-t0.tv_usec)*0.000001;
  double cput = clock() / (float)CLOCKS_PER_SEC;
  cout << "Real time elapsed:  " << t    << " s" << endl;
  cout << "CPU  time elapsed:  " << cput << " s" << endl;
  
  return 0;
  
}
